import type { SearchStoreState } from "#/search";
import { getAuthSearchResults } from "@/services/searchService";
import { defineStore } from "pinia";

export const useSearchStore = defineStore('Search', {
  state: (): SearchStoreState => ({
    toSearch: '',
    hits: {
      value: 0,
      relation: ''
    },
    params: {},
    results: [],
    pagination: {
      page: 1,
      pageSize: 10,
    },
    aggregations: {},
    buckets: {},
  }),
  getters: {
    q: (state) => `q=${state.toSearch}`,
    getParams: (state) => Object.values(state.params).reduce((acc, param) => `${acc}&${param.key}=${param.value}`, ''),
    page: (state) => state.pagination.page,
    pageSize: (state) => state.pagination.pageSize,
    total: (state) => state.hits.value,
    getPagination: (state) => `&page=${state.pagination.page}&page_size=${state.pagination.pageSize}`,
    aggregationBuckets() {
      return (name: string) => this.aggregations[name].buckets
    },
    aggregationBucketsAllChecked() {
      return (name: string) => {
        // console.log(name)
        return this.aggregations[name].buckets.reduce((acc, id) => acc && this.buckets[id].value, true)
      }
    },
    aggregationBucketsAllUnchecked() {
      return (name: string) => !this.aggregations[name].buckets.reduce((acc, id) => acc || this.buckets[id].value, false)
    },
    aggregationValue() {
      return (name: string) => {
        if (this.aggregationBuckets(name).length === 1) return this.buckets[this.aggregationBuckets(name)[0]].value ? `&${this.aggregations[name].filterName}=${this.buckets[this.aggregationBuckets(name)[0]].keyAsString || this.buckets[this.aggregationBuckets(name)[0]].key}`: ''
        const noValue = this.aggregationBucketsAllChecked(name) || this.aggregationBucketsAllUnchecked(name)
        return noValue ? '' : `&${this.aggregations[name].filterName}=${this.aggregationBuckets(name).map((id) => this.buckets[id]).filter((bucket) => bucket.value).map((bucket) => bucket.keyAsString || bucket.key).join(',')}`
      }
    },
    bucket() {
      return (id: string) => this.buckets[id]
    },
    bucketLabel() {
      return (id: string) => {
        const i18nBucketPath = `search.aggregation.bucket`
        const bucket = this.buckets[id]
        const { t, te } = this.i18n
        if (bucket.keyAsString) return `${t(`${i18nBucketPath}.boolean.${bucket.keyAsString}`)} (${bucket.docCount})`
        else return te(`${i18nBucketPath}.${bucket.key}`) ? `${t(`${i18nBucketPath}.${bucket.key}`, bucket.docCount)} (${bucket.docCount})` : `${bucket.key} (${bucket.docCount})`
      }
    },
    getAggregations() {
      return () => {
        return Object.entries(this.aggregations).reduce((acc, [name, aggregation]) => {
          return `${acc}${this.aggregationValue(name)}`
        }, '')
      }
    },
    query() {
      return `${this.q}${this.getParams}${this.getPagination}${this.getAggregations()}`
    },
  },
  actions: {
    resetPagination() {
      this.pagination = {
        page: 1,
        pageSize: 10,
      }
    },
    resetAggregations() {
      this.aggregations = {}
    },
    resetBuckets() {
      this.buckets = {}
    },
    async loadAuthResults(query?: string) {
      const { results, aggregations, hits } = await getAuthSearchResults(query || this.query)
      this.results = results.map((result) => {
        let subtitles, to, icon, color
        if (result.docType === 'project') {
          const { managers } = result
          const names = managers.join(', ')
          subtitles = [
            this.i18n.t('project.subtitles.managers', { names, label: this.i18n.t('project.managers', managers.length) }),
            this.i18n.t('set.count', result.setsCount),
            this.i18n.t('item.count', result.itemsCount),
          ]
          to = { name: 'projectSetsList', params: { projectId: result.id } }
          icon = 'mdi-ungroup'
          color = 'accent-blue'
        } else if (result.docType === 'set') {
          const {creator: {name}, itemsCount} = result
          subtitles = [
            this.i18n.t('set.initiator', { name }),
            this.i18n.t('item.count', itemsCount),
          ]
          to = { name: 'setItemsList', params: { setId: result.id} }
          icon = 'mdi-google-circles'
          color = 'accent-green'
        } else {
          to = { name: 'itemViewer', params: { itemId: result.id } }
          icon = 'mdi-google-circles-communities'
          color = 'accent-yellow'
        }

        const _result = {
          result: {
            id: result.id,
            title: result.title,
            description: result.description,
            thumbnail: result.thumbnail,
            chip: this.i18n.t(`${result.docType}.label`),
            subtitles,
          },
          to,
          icon,
          color,
        }
        return _result
      })

      if (!Object.keys(this.aggregations).length) {
        this.aggregations = {
          ...this.aggregations,
          ...Object.fromEntries(aggregations.filternames
            .map((aggregation) => {
              const name = Object.entries(aggregation)[0][0]
              const filterName = Object.entries(aggregation)[0][1]
              return [
                name,
                {
                  buckets: aggregations.aggregations[name].map((bucket) => {
                    const id = `${name}_${bucket.key}`
                    this.buckets[id] = {
                      ...bucket,
                      value: this.buckets[id]?.value ?? false,
                    }
                    return id
                  }),
                  filterName,
                  value: ''
                }
              ]
            })
          )
        }
      }
      
      this.hits = hits
    },
    setAllBucketsValueToFalse(name: string) {
      this.aggregations[name].buckets.forEach((id) => {
        this.buckets[id].value = false
      })
    }
  },
})
