import {describe, it, expect} from 'vitest'
import { messageToStringArray, getErrorDetail } from '@/services/alertService'
import { t } from '@/plugins/i18n'
import { AxiosError } from 'axios'
import type { InternalAxiosRequestConfig, AxiosResponse } from 'axios'

describe('functions', () => {
  describe('messageToArray', () => {
    it('should default to an empty array', () => {
      expect(messageToStringArray()).to.eql([])
    })
    it('should put string message into an array', () => {
      const msg = 'msg'
      expect(messageToStringArray(msg)).to.eql([msg])
    })
    it('should return a copy of a string array', () => {
      const msg = ['line1', 'line2']
      expect(messageToStringArray(msg)).to.eql(msg)
    })
    it('should return array values casted into string', () => {
      const msg = ['line1', '48']
      expect(messageToStringArray(msg)).to.eql(['line1', '48'])
    })
  })
  describe('getErrorDetail', () => {
    it('should return error casted to string', () => {
      class CustomError extends Error {
        constructor(message: string) {
          super(message)
          this.name = 'CustomError'
        }
      }
      const errorMessage = 'Oh lala'
      const error = new CustomError(errorMessage)
      expect(getErrorDetail(error))
          .to.eql([`${CustomError.name}: ${errorMessage}`])
    })
    describe('axios errors', () => {
      it('should return timeout message with only request data', () => {
        const error = new AxiosError('', '504', {} as InternalAxiosRequestConfig, {})
        expect(getErrorDetail(error))
            .toEqual([t('error.timeout')])
      })
      describe('error with response', () => {
        function createAxiosError(responseData: any, status=400, statusText='Not found'): AxiosError {
          return new AxiosError(
              '',
              status.toString(),
              {} as InternalAxiosRequestConfig,
              {},
              {
                status,
                data: responseData,
                statusText,
              } as AxiosResponse,
          )
        }
        it('should return status code as first line', () => {
          const status = 404
          const error = createAxiosError({}, status)
          expect(getErrorDetail(error)[0])
              .toEqual(t('error.response', {status}))
        })
        it('should return error detail from data id data is an array', () => {
          const data = ['could not wake up', 'this could be bad']
          const error = createAxiosError(data)
          expect(getErrorDetail(error).slice(1))
              .toEqual(data)
        })
        it('should return error detail from detail data key', () => {
          const data = { detail: 'could not wake up' }
          const error = createAxiosError(data)
          expect(getErrorDetail(error)[1])
              .toEqual(data.detail)
        })
        it('should return error detail from error data key', () => {
          const data = { error: 'this could be bad' }
          const error = createAxiosError(data)
          expect(getErrorDetail(error)[1])
              .toEqual(data.error)
        })
        it('should return error detail from errors data key', () => {
          const data = { errors: ['could not wake up', 'this could be bad'] }
          const error = createAxiosError(data)
          expect(getErrorDetail(error).slice(1))
              .toEqual(data.errors)
        })
        it('should default return statusText', () => {
          const statusText = 'Payment Required'
          const error = createAxiosError({}, 402, statusText)
          expect(getErrorDetail(error)[1])
              .toEqual(statusText)
        })
      })
    })
  })
})
