import { describe, it, expect, beforeEach, afterEach, vi } from 'vitest'
import { createPinia, setActivePinia } from 'pinia'

import { authAxios } from '@/services/authService'
import { notifyError, notifySuccess } from '@/services/alertService'
import { useSetStore, initSet } from '@/stores/set'
import { useProjectStore } from '@/stores/project'
import { t } from '@/plugins/i18n'

import type { Mocked } from 'vitest'
import type { ProjectSet } from '#/set'
import type { Authority } from '@/services/authService'


vi.mock(
    '@/services/alertService',
    () => ({
      notifyError: vi.fn(),
      notifySuccess: vi.fn()
    })
)

vi.mock(
    '@/services/authService',
    () => ({
      authAxios: {
        delete: vi.fn(),
        get: vi.fn(),
        patch: vi.fn(),
        post: vi.fn(),
      },
    })
)

const mockedAxios: Mocked<Authority> = vi.mocked(authAxios, true)

const initProjectSet = (): ProjectSet => ({
  canContribute: false,
  description: '',
  id: '',
  isPublic: false,
  numItems: false,
  templateId: '0',
  title: '',
  thumbnails: {},
  updatedAt: '2022'
})

describe('store/set', () => {
  setActivePinia(createPinia())
  const store = useSetStore()
  const projectStore = useProjectStore()

  describe('actions', () => {
    const axiosError = new Error('failed')
    const testError = (apiMethod: 'get' | 'patch' | 'post' | 'delete', message: string, storeAction: Function, ...params: any[]) =>
        async () => {
          mockedAxios[apiMethod].mockRejectedValueOnce(axiosError)
          await storeAction(...params)
          expect(notifyError).toHaveBeenCalledOnce()
          expect(notifyError).toHaveBeenLastCalledWith(
              message,
              axiosError
          )
        }
    const testSuccess = (message: string) => {
      expect(notifySuccess).toHaveBeenNthCalledWith(1, message)
    }
    const projectId = '48e9-6a97'

    const retrieveSet = {
      id: '987a',
      title: 'set title',
      description: 'au moins un mot',
      templateId: '21c5',
      publisher: 'someone',
      isPublic: false
    }
    beforeEach(() => {
      store.$reset()
      projectStore.current.id = projectId
    })
    afterEach(() => {
      vi.restoreAllMocks()
    })
    describe('createSet', () => {
      const newSet = {
        title: 'set title',
        description: 'au moins un mot',
        fromTemplateId: '21c5',
        publisher: 'someone',
        isPublic: false
      }
      beforeEach(async () => {
        mockedAxios.post.mockResolvedValueOnce({data: retrieveSet})
        await store.createSet(newSet)
      })
      it('should post set and projectId to api', () => {
        expect(mockedAxios.post).toHaveBeenCalledOnce()
        expect(mockedAxios.post).toHaveBeenLastCalledWith(
            'sets/',
            {projectId, ...newSet}
        )
      })
      it('should update current set with retrieve data', async () => {
        expect(store.current).toEqual(retrieveSet)
      })
    })
    describe('loadSet', () => {
      it('should call set detail api', () => {
        const setId = '879e'
        store.loadSet(setId)
        expect(mockedAxios.get).toHaveBeenNthCalledWith(1, 'sets/879e/')
      })
      it('should update current set with retrieve data', async () => {
        mockedAxios.get.mockResolvedValueOnce({data: retrieveSet})
        await store.loadSet('874c')
        expect(store.current).toEqual(retrieveSet)
      })
      it('should notify explicit error', testError(
          'get',
          t('notify.error.set.load'),
          store.loadSet, '874c'
      ))
    })
    describe('loadProjectSets', () => {
      const setList = [
        {id: 'e321-47bd', name: 'set one'},
        {id: 'e321-47be', name: 'set two'},
        {id: 'e321-47bf', name: 'set four'},
      ]
      describe('with default values', () => {
        beforeEach(async () => {
          mockedAxios.get.mockResolvedValueOnce({
            data: {count: 3, nextPage: null, results: setList}
          })
          await store.loadProjectSets()
        })
        it('should call retrieve list api', () => {
          expect(mockedAxios.get).toHaveBeenNthCalledWith(
              1,
              `sets/?project_id=${projectId}&page=1&page_size=0`
          )
        })
        it('should populate project set list with retrieve data', () => {
          expect(store.projectSets).toEqual(setList)
        })
      })
      it('should notify on failure', testError(
          'get',
          t('notify.error.set.all'),
          store.loadProjectSets
      ))
    })
    describe('updateSet', () => {
      const patchedSet = {
        id: '2131-543c',
        title: 'set title',
        description: 'au moins un mot',
        templateId: '21c5',
        publisher: 'someone',
      }
      const patchData = { title: 'new title' }
      const retrieveData = {
        ...initSet(),
        ...patchedSet,
        ...patchData,
      }
      beforeEach(async () => {
        mockedAxios.patch.mockResolvedValueOnce({data: retrieveData})
        store.current = {
          ...initSet(),
          ...patchedSet
        }
        await store.updateSet(patchData)
      })
      it('should patch set and projectId to api', () => {
        expect(mockedAxios.patch).toHaveBeenCalledOnce()
        expect(mockedAxios.patch).toHaveBeenLastCalledWith(
            'sets/2131-543c/',
            patchData
        )
      })
      it('should update current set with retrieve data', () => {
        expect(store.current).toEqual(retrieveData)
      })
    })
    describe('deletion', () => {
      const rubbishSetId = '32d3-5478'
      describe('deleteSet', () => {
        describe('success scenario', () => {
          beforeEach(async () => {
            mockedAxios.delete.mockResolvedValueOnce({})
            store.current = {
              ...initSet(),
              id : rubbishSetId
            }
            await store.deleteSet(rubbishSetId)
          })
          it('should call delete api', () => {
            expect(mockedAxios.delete)
                .toHaveBeenNthCalledWith(1, `sets/${rubbishSetId}/`)
          })
          it('should reset current set', () => {
            expect(store.current).toEqual(initSet())
          })
          it('should notify that success', () => {
            testSuccess(t('notify.delete.success'))
          })
        })
        it('should notify explicit error', testError(
            'delete',
            t('notify.error.set.delete', 1),
            store.deleteSet, rubbishSetId
        ))
      })
      describe('deleteMultiSet', () => {
        describe('success scenario', () => {
          beforeEach(async () => {
            mockedAxios.delete.mockResolvedValueOnce({})
            store.projectSets = [{...initProjectSet(), id: rubbishSetId}, initProjectSet()]
            await store.deleteMultiSet([rubbishSetId])
          })
          it('should call delete api', () => {
            expect(mockedAxios.delete).toHaveBeenCalledOnce()
            expect(mockedAxios.delete).toHaveBeenLastCalledWith(
                'sets/',
                {data: {setIds: [rubbishSetId]}}
            )
          })
          it('should remove list from project sets', async () => {
            expect(store.projectSets).to.have.lengthOf(1)
          })
          it('should notify that success', () => {
            testSuccess(t('notify.delete.success'))
          })
        })
        it('should notify explicit error', testError(
            'delete',
            t('notify.error.set.delete', 2),
            store.deleteMultiSet, ['','']
        ))
      })
    })
  })
})

