import { describe, it, expect, beforeEach, afterEach, vi } from 'vitest'

import { authAxios } from '@/services/authService'
import { notifyError, notifySuccess} from '@/services/alertService'
import { createPinia, setActivePinia } from 'pinia'
import { useTemplateStore, getPrivateFields, TEMPLATE_SCOPE } from '@/stores/template'
import { useProjectStore } from '@/stores/project'
import { t } from '@/plugins/i18n'

import type { Mocked } from 'vitest'
import type { Authority } from '@/services/authService'
import type { UsfField, PountTemplate } from '#/template'

vi.mock(
    '@/services/alertService',
    () => ({
      notifyError: vi.fn() ,
      notifySuccess: vi.fn()
    })
)


vi.mock(
    '@/services/authService',
    () => ({
      authAxios: {
        delete: vi.fn(),
        get: vi.fn(),
        patch: vi.fn(),
        post: vi.fn(),
      },
    })
)

const mockedAxios: Mocked<Authority> = vi.mocked(authAxios, true)

describe('store/template', () => {
  describe('getPrivateFields', () => {
    it('should not return id of field that has no name', () => {
      const privateList = getPrivateFields([{id: 'fhqrh'} as UsfField])
      expect(privateList).toHaveLength(0)
    })
    it('should return id of field that is not visible', () => {
      const fields = [{id: 'fhqrh', name: 'creator', visible: false} as UsfField]
      const privateList = getPrivateFields(fields)
      expect(privateList).toEqual(fields.map(f => f.id))
    })
    it('should return id of field that must not exist', () => {
      const fields = [{id: 'fhqrh', name: 'creator', exist: false} as UsfField]
      const privateList = getPrivateFields(fields)
      expect(privateList).toEqual(fields.map(f => f.id))
    })
    it('should return id of field if it\'s parent is already private', () => {
      const fields = [{id: 'fhqrh', name: 'creator'} as UsfField]
      const privateList = getPrivateFields(fields, true)
      expect(privateList).toEqual(fields.map(f => f.id))
    })
    it('should return id of field nested in layout field', () => {
      const fields = [{
        id: 'fhqrh',
        fields: [{ id: 'nested', name:'type', visible: false }]
      }  as UsfField]
      const privateList = getPrivateFields(fields, false)
      expect(privateList).toEqual(['nested'])
    })
  })
  describe('actions', () => {
    setActivePinia(createPinia())
    const store = useTemplateStore()
    const projectStore = useProjectStore()
    const projectId = 'e23b-32a4'

    const axiosError = new Error('failed')

    beforeEach(() => {
      store.$reset()
      projectStore.current.id = projectId
    })
    afterEach(() => {
      vi.restoreAllMocks()
    })
    describe('createTemplate', ()=> {
      const templateData: Omit<PountTemplate, 'id' | 'scope' | 'privateFields'> = {
        name: 'new template',
        fields: [],
        messages : { defaultLocale: 'fr', locales: {fr: {}}},
        creator: {id: '48ae'}
      }
      const createdTemplate = {
        id: '3215',
        ...templateData,
        scope: TEMPLATE_SCOPE.PROJECT,
        privateFields: []
      }
      it('should post template data, scope, and project id to templates api', async () => {
        mockedAxios.post.mockResolvedValueOnce({data: createdTemplate})
        await store.createTemplate(templateData)
        expect(mockedAxios.post).toHaveBeenCalledOnce()
        expect(mockedAxios.post).toHaveBeenLastCalledWith(
          'templates/',
          {
            projectId,
            scope: TEMPLATE_SCOPE.PROJECT,
            ...templateData,
            privateFields: []
          }
        )
      })
      it('should add retrieved new template to project\'s templates', async () => {
        mockedAxios.post.mockResolvedValueOnce({data: createdTemplate})
        await store.createTemplate(templateData)
        expect(store.projectList).toHaveLength(1)
        expect(store.projectList[0].id).toEqual('3215')
      })
      it('should notify explicit error', async () => {
        mockedAxios.post.mockRejectedValueOnce(axiosError)
        await store.createTemplate(templateData)
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.template.create'),
            axiosError
        )
      })
    })
    describe('loadProjectTemplates', ()=> {
      it('should call template api for current project', () => {
        store.loadProjectTemplates()
        expect(mockedAxios.get).toHaveBeenNthCalledWith(1,`projects/${projectId}/templates/`)
      })
      it('should load project\'s template list', async () => {
        const projectTemplates = [
          {id: '123a', name: 'Template 1', fields: [], messages: {}, scope: TEMPLATE_SCOPE.PROJECT},
          {id: '456b', name: 'Template 2', fields: [], messages: {}, scope: TEMPLATE_SCOPE.SET},
        ]
        mockedAxios.get.mockResolvedValueOnce({data: projectTemplates})
        await store.loadProjectTemplates()
        expect(store.projectList).toEqual(projectTemplates)
      })
      it('should notify explicit error', async () => {
        mockedAxios.get.mockRejectedValueOnce(axiosError)
        await store.loadProjectTemplates()
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.template.all'),
            axiosError
        )
      })
    })
    describe('loadPublicTemplates', ()=> {
      it('should call public templates api', () => {
        store.loadPublicTemplates()
        expect(mockedAxios.get).toHaveBeenNthCalledWith(1,`templates/`)
      })
      it('should update public template list', async () => {
        const publicTemplates = [
          {id: '123a', name: 'Template 1', fields: [], messages: {}, scope: TEMPLATE_SCOPE.BASE},
          {id: '456b', name: 'Template 2', fields: [], messages: {}, scope: TEMPLATE_SCOPE.GLOBAL},
        ]
        mockedAxios.get.mockResolvedValueOnce({data: publicTemplates})
        await store.loadPublicTemplates()
        expect(store.publicList).toEqual(publicTemplates)
      })
      it('should notify explicit error', async () => {
        mockedAxios.get.mockRejectedValueOnce(axiosError)
        await store.loadPublicTemplates()
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.template.all'),
            axiosError
        )
      })
    })
    describe('loadTemplatesList', ()=> {
      it('should call project`s template list api', () => {
        store.loadTemplatesList()
        expect(mockedAxios.get).toHaveBeenNthCalledWith(1,`projects/${projectId}/list_templates/`)
      })
      it('should update template list', async () => {
        const templatesForSet = [
          {id: '123a', name: 'Template 1', scope: TEMPLATE_SCOPE.BASE},
          {id: '456b', name: 'Template 2', scope: TEMPLATE_SCOPE.GLOBAL},
          {id: 'e234', name: 'Template 3', scope: TEMPLATE_SCOPE.PROJECT},
          {id: 'f5fb', name: 'Template 4', scope: TEMPLATE_SCOPE.SET},
        ]
        mockedAxios.get.mockResolvedValueOnce({data: templatesForSet})
        await store.loadTemplatesList()
        expect(store.templateList).toEqual(templatesForSet)
      })
      it('should notify explicit error', async () => {
        mockedAxios.get.mockRejectedValueOnce(axiosError)
        await store.loadTemplatesList()
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.template.all'),
            axiosError
        )
      })
    })
    describe('loadTemplate', ()=> {
      const templateId = '7654'
      const retrieveTemplate = {
        id: templateId,
        name: 'my template',
        fields: [{
          id: "5vxode5mjsh",
          name: "title",
          type: "text",
          exist: true,
          label: "Title",
          visible: true,
          required: true,
          component: "UsfText",
        }],
        messages: {
          default: 'fr',
          locales: {fr: {"5vxode5mjsh": {label: "Titre"}}}
        },
        scope: TEMPLATE_SCOPE.SET
      }
      it('should call template detail api', () => {
        store.loadTemplate(templateId)
        expect(mockedAxios.get).toHaveBeenNthCalledWith(1,`templates/${templateId}/`)
      })
      it('should update current template', async () => {
        mockedAxios.get.mockResolvedValueOnce({data: retrieveTemplate })
        await store.loadTemplate(templateId)
        expect(store.current).toEqual(retrieveTemplate)
      })
      it('should notify explicit error', async () => {
        mockedAxios.get.mockRejectedValueOnce(axiosError)
        await store.loadTemplate(templateId)
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.template.load'),
            axiosError
        )
      })
    })
    describe('updateTemplate', ()=> {
      const patchData = {
        name: 'patched',
        fields: []
      }
      const templateId = '7654'
      const initialTemplate: PountTemplate = {
        id: templateId,
        creator: 'IA',
        fields: [{
          component: "UsfText",
          exist: true,
          id: "5vxode5mjsh",
          label: "Title",
          name: "title",
          parentId: "root",
          required: true,
          type: "text",
          visible: true,
        }],
        name: "initial",
        messages: {
          defaultLocale: 'fr',
          locales: {fr: {"5vxode5mjsh": {label: "Titre"}}}
        },
        privateFields: [],
        scope: TEMPLATE_SCOPE.SET,
      }
      const retrieveTemplate = {
        ...initialTemplate,
        name: 'patched',
      }
      it('should call template\'s patch api', () => {
        store.updateTemplate({id: templateId, ...patchData})
        expect(mockedAxios.patch).toHaveBeenCalledOnce()
        expect(mockedAxios.patch).toHaveBeenLastCalledWith(
          `templates/${templateId}/`,
          { ...patchData, privateFields: []}
        )
      })
      it('should update current template', async () => {
        store.current = { ...initialTemplate }
        mockedAxios.patch.mockResolvedValueOnce({data: retrieveTemplate})
        await store.updateTemplate({id: '', ...patchData})
        expect(store.current).toEqual(retrieveTemplate)
      })
      it('should notify update success', async () => {
        mockedAxios.patch.mockResolvedValueOnce({data: retrieveTemplate})
        await store.updateTemplate({id: '', ...patchData})
        expect(notifySuccess).toHaveBeenNthCalledWith(1, t('notify.updated.template'))
      })
      it('should notify explicit error', async () => {
        mockedAxios.patch.mockRejectedValueOnce(axiosError)
        await store.updateTemplate({id: '', ...patchData})
        expect(notifyError).toHaveBeenCalledOnce()
        expect(notifyError).toHaveBeenLastCalledWith(
            t('notify.error.update'),
            axiosError
        )
      })
    })
  })
})
